<?php
/*
 * Daemon responsible for making changes in system configuration.
 * It receives commands and returns answers via a tcp-socket.
 *
 */

// Make sure all errors are displayed
error_reporting(E_ALL);
ini_set("display_errors", "On");

ini_set("max_execution_time", "0");
ini_set("max_input_time", "0");
set_time_limit(0);

declare(ticks = 1);


# replace fds
fclose(STDIN);
fclose(STDOUT);
fclose(STDERR);
$stdin = fopen('/dev/null', 'r'); // set fd/0
$stdout = fopen('/dev/null', 'w'); // set fd/1
$stderr = fopen('php://stdout', 'w'); // hack to duplicate fd/1 to 2

require_once("sockets.inc.php");
require_once("daemon.inc.php");
require_once("socket-daemon.inc.php");
require_once("logger.inc.php");
require_once("/opt/op5sys/etc/config-daemon.conf");
require_once("protocol.inc.php");

global $CONF;

Logger::setSyslog($CONF["ENABLE_SYS_LOG"], "op5Config");
Logger::setFile($CONF["ENABLE_LOG_FILE"], $CONF["LOG_FILE"]);
Logger::setStderr($CONF["ENABLE_STDERR_OUTPUT"]);
Logger::debugLevel(Logger::LOG);

class ConfigDaemon extends SocketDaemon
{
	function __construct()
	{
		parent::__construct("op5Config", "op5Config", 1);
	}

	function acceptSocket($cs)
	{
		$prot = new Protocol();

		global $CONF;
		$pid = posix_getpid();
		Logger::debug("Server open connection");
		Logger::debug("Child... PID = %d", $pid);

		$counter = 0;
		do
		{
			$err = $cs->read_timeout($cmd, 1000, PHP_NORMAL_READ);
			if ($cmd != '')
			{
				$counter = 0;
				$feedback = $prot->execCmd($cmd);
			}
			else
			{
				$counter++;
				if ($counter > $CONF["ROLLBACK_TIMEOUT"] || $err === false)
				{
					$feedback = $prot->rollbackOnTimeout();
				}
				else
				{
					$feedback = false;
				}
			}

			if(is_string($feedback))
				$cs->write_timeout($feedback, 10000);
		}
		while ($err !== false && $counter < $CONF["SESSION_TIMEOUT"]);
		Logger::debug("(%d) Server close connection", $pid);
	}

	function rejectSocket($cs)
	{
		$pid = posix_getpid();
		Logger::debug("Server open connection");
		Logger::debug("Child... PID = %d", $pid);
		Logger::debug("Too many simultaneous connections");
		$err = $cs->write_timeout("255|Too many simultaneous connections\0");
		Logger::debug("(%d) Server close connection", $pid);
	}

}

$qd = new ConfigDaemon();
$qd->run();

exit(0);
