<?php

if(phpversion() < '5.3')
{
	define_syslog_variables();
}

class LogFile
{
	public function __construct($name, $mode)
	{
		$this->file = fopen($name, $mode);
	}

	public function __destruct()
	{
		fclose($this->file);
	}

	public function logString($str)
	{
		static $pid;
		if(!$pid) {
			$pid = posix_getpid();
		}
		fwrite($this->file, sprintf("[%d] %s", $pid, $str));
	}

	public function isValid()
	{
		return !is_null($this->file);
	}
}

class Logger
{
	public static $level = 0;

	private static $syslogState = true;
	private static $filelogState = false;
	private static $stderrlogState = false;
	private static $filelog = NULL;

	const LOG = 0;
	const DEBUG = 1;

	public static function setSyslog($enabled, $ident = NULL)
	{
		Logger::$syslogState =  $enabled;

		if (!$enabled)
			closelog();
		else
			if ($ident)
			{
				closelog();
				openlog($ident, LOG_NDELAY, LOG_DAEMON);
			}
	}

	public static function setFile($enabled, $name)
	{
		Logger::$filelogState =  $enabled;

		if ($enabled)
			Logger::$filelog = new LogFile($name, "a+");
		else
			Logger::$filelog = NULL;

	}

	public static function setStderr($enabled)
	{
		Logger::$stderrlogState =  $enabled;
	}

	public static function debugLevel($level)
	{
		Logger::$level = $level;
	}

	public static function debug()
	{
		if(Logger::$level == Logger::DEBUG)
		{
			$args = func_get_args();
			call_user_func_array( array("Logger", "log"), $args );
		}
	}

	public static function log()
	{
		$args = func_get_args();
		$msg = call_user_func_array( "sprintf", $args ) . "\n";

		if (Logger::$stderrlogState)
			echo $msg;

		if (Logger::$syslogState)
			if (Logger::$level == Logger::DEBUG)
				syslog(LOG_DEBUG, $msg);
		else
			syslog(LOG_WARNING, $msg);

		if (Logger::$filelogState && Logger::$filelog->isValid())
			Logger::$filelog->logString($msg);

	}
}

?>
