<?php
/*
 * Daemon responsible for perform of some commands. It receives command and
 * returns answers via a tcp-socket.
 *
 */

require_once("sockets.inc.php");
require_once("daemon.inc.php");
require_once("logger.inc.php");

class SocketDaemon extends Daemon
{
	var $socket;
	var $child_pids_accept = array();
	var $child_pids_reject = array();

	var $domain;
	var $type;
	var $protocol;

	var $address;
	var $port;
	var $chld;

	var $maxConnections;

	function __construct($programName, $syslogIdent, $maxConnections, $domain = AF_UNIX, $type = SOCK_STREAM, $protocol = 0, $address = "/tmp/op5daemon", $port = 0)
	{
		parent::__construct($programName, $syslogIdent);

		$this->domain = $domain;
		$this->type = $type;
		$this->protocol = $protocol;
		$this->address = $address;
		$this->port = $port;
		$this->maxConnections = $maxConnections;

		if ($domain == AF_UNIX && file_exists($address))
			unlink($address);
	}

	function initsocket()
	{
		Logger::debug("Create socket");
		$this->socket = socket_create($this->domain, $this->type, $this->protocol);
		if(!$this->socket){
			Logger::log("Socket create error." . socket_strerror(socket_last_error()));
			exit;
		}

		/* set socket receive timeout to 1 second */
		// socket_set_option($sock, SOL_SOCKET, SO_RCVTIMEO, array("sec" => 1, "usec" => 0));

		/* socket reuse adress to get rid of error unable to bind, address already in use */
		if (!socket_set_option($this->socket, SOL_SOCKET, SO_REUSEADDR, 1)) {
			Logger::log(socket_strerror(socket_last_error($this->socket)));
			exit;
		}

		Logger::debug("Socket bind: %s : %d", $this->address , $this->port);
		if (! socket_bind($this->socket, $this->address, $this->port)) {
			Logger::log(socket_strerror(socket_last_error($this->socket)));
			exit;
		}

		Logger::debug("Socket listen");
		if (! socket_listen($this->socket)) {
			Logger::log(socket_strerror(socket_last_error($this->socket)));
			exit;
		}

		if ($this->domain == AF_UNIX)
		{
			$is_suse = file_exists('/etc/SuSE-release');
			if($is_suse) {
				if(!chgrp($this->address, "www")) {
					Logger::log("Could not change group on socket: ".$this->address);
				}
			} else {
				if(!chgrp($this->address, "apache")) {
					Logger::log("Could not change group on socket: ".$this->address);
				}
			}
			if(!chmod($this->address, 0660)) {
				Logger::log("Could not change group on socket: ".$this->address);
			}
		}

	}

	function acceptSocket($cs)
	{
	}

	function rejectSocket($cs)
	{
	}

	function daemonMain()
	{
		$this->initsocket();
		Logger::debug("Daemon started");

		pcntl_signal(SIGCHLD, SIG_IGN);
		while (! $this->getShouldExit()) {

			$r = array($this->socket);
			$w = array($this->socket);
			$e = array();
			$num_changed_sockets = @socket_select($r, $w, $e, 10);

			if ($num_changed_sockets === FALSE) {
				Logger::log("Error on socket_select");
				break;
			} else if ($num_changed_sockets < 1) {
				continue;
			}

			$client_socket = socket_accept($this->socket);
			pcntl_waitpid_noblock($this->child_pids_accept);
			pcntl_waitpid_noblock($this->child_pids_reject);

			$pid = pcntl_fork();
			if ($pid == -1) {
				Logger::log("Could not fork child!");
				exit;
			} else if ($pid) {
				// we are the parent
				if(count($this->child_pids_accept) < $this->maxConnections)
					$this->child_pids_accept[] = $pid;
				else
					$this->child_pids_reject[] = $pid;
			} else {
				// we are the child
				pcntl_signal(SIGCHLD, SIG_DFL);

				if(count($this->child_pids_accept) < $this->maxConnections)
					$this->acceptSocket(new ClientSocket($client_socket));
				else
					$this->rejectSocket(new ClientSocket($client_socket));
				Logger::debug("Client thread exit!!!!");
				exit;
			}
		}

		Logger::debug("Shutdown socket");
		while (count($this->child_pids_accept) > 0) {
			pcntl_waitpid_noblock($this->child_pids_accept);
			sleep(1);
		}

		while (count($this->child_pids_reject) > 0) {
			pcntl_waitpid_noblock($this->child_pids_reject);
			sleep(1);
		}

		Logger::debug("Closing socket");
		socket_shutdown($this->socket, 2);
		socket_close($this->socket);
	}
}

function pcntl_waitpid_noblock(& $pid_array)
{
	$retval = array();

	foreach ($pid_array as $pid) {
		$status = 0;
		if (!pcntl_waitpid($pid, $status, WNOHANG))
		{
			$retval[] = $pid;
		}
	}

	$pid_array = $retval;

	Logger::debug("Number of children still alive: %d.", count($pid_array));
}

?>
