<?php

require_once("logger.inc.php");

function ms_to_s_ms($totalMs, & $seconds, &$ms)
{
	$seconds = $totalMs / 1000;
	$ms = $totalMs - (1000 * intval($totalMs / 1000));
}

function socket_write_timeout(& $sock, $string, $timeout = 5)
{
	while (true) {
		$f = $r = array();
		$w = array($sock);
		$num_sockets = socket_select($r, $w, $f, $timeout);

		if ($num_sockets === false)
			return false;

		if ($num_sockets == 0)
			return 0;

		#if (count($w) < 1) {
		#	// This must be an error..
		#	return FALSE;
		#}

		$len = strlen($string);
		$bytes_written = socket_write($sock, $string);
		if ($bytes_written === false)
			return false;

		if ($bytes_written < $len) {
			$string = substr($string, $bytes_written, $len - $bytes_written);
			Logger::debug("Bytes left %d", strlen($string) + 1);
		} else {
			Logger::debug("Wrote %d bytes, len %d bytes", $bytes_written, $len);
			return 1;
		}
	}
}

class ClientSocket
{
	function __construct(& $sock = null)
	{
		if ($sock) {
			$this->sock = $sock;
		} else {
			$this->sock = socket_create(AF_INET, SOCK_STREAM, SOL_TCP);
		}
	}

	function __destruct()
	{
		//socket_shutdown($this->sock, 2);
		socket_close($this->sock);
	}

	function connect($host, $port)
	{
		if (false === socket_connect($this->sock, $host, $port)) {
			return false;
		}
		return true;
	}

	function last_error()
	{
		return socket_last_error($this->sock);
	}

	function last_error_string()
	{
		return socket_strerror($this->last_error());
	}

	function log($message)
	{
		Logger::log($message);
	}

	function callback()
	{

	}

	function write_timeout($string, $totalTimeout = 5000)
	{
		return socket_write_timeout($this->sock, $string, $totalTimeout);
	}

	function read_timeout(& $retval, $timeoutInMs = 100, $readType = PHP_BINARY_READ)
	{
		$retval = '';
		while (true) {
			ms_to_s_ms($timeoutInMs, $sec, $ms);

			$r = array($this->sock);
			$w = $f = array();
			$num_sockets = socket_select($r, $w, $f, $sec, $ms);

			if ($num_sockets === false)
			  return false;

			if ($num_sockets > 0) {
				if (false === ($sock_retval = socket_read($this->sock, 512, $readType))) {
					return false;
				} else {
					if (empty($sock_retval))
						return false;
					$retval .= $sock_retval;
					return 1;
				}
			} else if (count($f) > 0) {
				// some failure..
				return false;
			} else {
				// This must be a timeout
				Logger::debug("socket_read_timeout(): got timeout");
				return 0;
			}
		}
	}

	function read_until_delimiter(& $value, $delim, $timeoutInMs = 5000)
	{
		$retval = "";
		$start = microtime(true);

		while (true) {
			$retval = $this->read_timeout($part, 1000);
			if (false === $retval)
			  return false;

			if (0 == $retval) {
				$this->callback();
			}
			else {
				if (contains_delimiter($part, chr(0))) {
					$value .= $part;
					return true;
				}
				$value .= $part;
			}

			if (($start + ($timeoutInMs / floatval(1000))) < microtime(true)) {
				Logger::debug("read_until_delimiter() got real timeout");
				return false;
			}
		}
	}
}

function socket_read_loop(& $socket, & $value)
{
	$retval = '';
	while (true) {
		// TODO: Switch to socket_accept before read to make sure we don't block.
		// Then we can also have a timeout on the reading
		Logger::debug("before socket_read");
		if (FALSE === ($sock_retval = socket_read($socket, 512, PHP_BINARY_READ))) {
			return $sock_retval;
		}
		Logger::debug("after socket_read");
		if (contains_delimiter($sock_retval, chr(0))) {
			$retval .= $sock_retval;
			return $retval;
		}
		$retval .= $sock_retval;
	}
}

// Checks if $string contains character $delim. If so, returns true
// and modifies $string to not include the delimiter.
function contains_delimiter(& $string, $delim)
{
	if (FALSE !== ($pos = strpos($string, $delim))) {
		$string = substr($string, 0, $pos);
		return true;
	}
	return false;
}

?>
