<?php

class Settings
{
	private $socket = null;
	private $transaction = null;

	public function __construct()
	{
		$this->socket = socket_create(AF_UNIX, SOCK_STREAM, 0);
		if (!$this->socket || !socket_connect($this->socket, '/tmp/op5daemon', 0)) {
			$err = socket_last_error();
			throw new Exception("Connection failed [$err]: ".socket_strerror($err));
		}
		$this->transaction = null;
	}

	public function __destruct()
	{
		socket_shutdown($this->socket, 2);
		socket_close($this->socket);
	}

	public function read($name, $action, $value = '')
	{
		// Note: socket_write can write only some portion of data,
		// but let's assume that everything is written if there is no error
		if (socket_write($this->socket, "read {$name} {$action} {$value}\n") === false)
			throw new Exception('Failed to read settings');

		$reply = socket_read($this->socket, 10240);
		// large buffer is needed for timezones and yum updates output
		if ($reply === false)
			throw new Exception('Failed to read settings');

		list($exitcode, $message) = explode('|', trim($reply)); //trim() is used to get rid of \0 at the end
		if ($exitcode != '0')
			throw new Exception('Failed to read settings. ' . $message);

		return $message;
	}

	public function edit($name, $action, $option, $value = '')
	{
		if ($this->transaction !== null)
		{
			$this->transaction .= "edit {$name} {$action} {$option} {$value}\n";
			return;
		}

		// Note: socket_write can write only some portion of data,
		// but let's assume that everything is written if there is no error
		if (socket_write($this->socket, "edit {$name} {$action} {$option} {$value}\n") === false)
			throw new Exception('Failed to change settings');

		$reply = socket_read($this->socket, 10240);
		if ($reply === false)
			throw new Exception('Failed to change settings');

		list($exitcode, $message) = explode('|', $reply);
		if ($exitcode != '0')
			throw new Exception('Failed to change settings. ' . $message);
	}

	public function apply($name)
	{
		if ($this->transaction !== null)
		{
			$this->transaction .= "edit {$name} apply\n";
			return;
		}

		// Note: socket_write can write only some portion of data,
		// but let's assume that everything is written if there is no error
		if (socket_write($this->socket, "edit {$name} apply\n") === false)
			throw new Exception('Failed to apply changes');

		if (socket_read($this->socket, 1024) === false)
			throw new Exception('Failed to apply changes');
	}

	public function begin()
	{
		/*
		if (socket_write($this->socket, "begin\n") === false)
			throw new Exception('Failed to initiate transaction');
		*/
		$this->transaction = "begin\n";
	}

	public function end()
	{
		$this->transaction .= "end\n";
		if (socket_write($this->socket, $this->transaction) === false)
			throw new Exception('Failed to change settings. Failed to start transaction');

		$reply = socket_read($this->socket, 10240);
		if ($reply === false)
			throw new Exception('Failed to change settings. Failed to finish transaction');

		list($exitcode, $message) = explode('|', $reply);
		if ($exitcode != '0')
			throw new Exception('Failed to change settings. ' . $message);

		$this->transaction = null;
		if (socket_write($this->socket, "commit\n") === false)
			throw new Exception('Failed to change settings. Failed to finish transaction');
	}
}

?>
