<?php defined('SYSPATH') OR die('No direct access allowed.');

class Backup_Model
{

	protected $_config = NULL;

	public function __construct() {
		$this->_config = new Settings();
	}

	public function read()
	{
		$config = $this->_config;
		$settings = array(
			'transfer' => 'transfer',
			'storagepath' => 'storagepath',
			'server'  => 'ftp_server',
			'serverpath' => 'ftp_path',
			'user' => 'ftp_user',
			'password' => 'ftp_pass',
			'datedirs' => 'createftpdatedirs'
		);

		$reply = array();
		foreach ($settings as $key => $setting)
			$reply[$key] = $config->read('backup', 'settings', $setting);

		$schedule = $config->read('backup', 'schedule');
		$time = array();
		if ($schedule != '')
		{
			list($time['minute'], $time['hour'], $time['day'], $time['month'], $time['dayofweek']) = explode(' ', $schedule);
			if ($time['dayofweek'] != '*')
				$reply['frequency'] = 'weekly';
			elseif ($time['month'] != '*')
				$reply['frequency'] = 'yearly';
			elseif ($time['day'] != '*')
				$reply['frequency'] = 'monthly';
			elseif ($time['hour'] != '*')
				$reply['frequency'] = 'daily';
			else
				throw new Exception('Cannot read backup schedule: wrong format.');

			$reply['time'] = $time['hour'] . ':' . $time['minute'];
		}
		else
		{
			$reply['frequency'] = 'never';
			$reply['time'] = '';
		}

		return $reply;
	}

	public function get_backup_variants($path) {
		$config = $this->_config;

		$res = explode("\n", $config->read('backup/files', $path));
		$file_res = array();
		foreach ($res as $key => $value) {
			$file_res[$value] = array_pop(explode("/", $value));
		}
		return $file_res;
	}

	public function restore($file_path, $files_only = false) {
		$this->_config->edit('backup/files', $file_path, $files_only ? "-f" : "");
	}

	public function edit($transfer, $storagepath, $server, $serverpath, $user, $password, $datedirs, $frequency, $time)
	{
		$config = $this->_config;

		$config->edit('backup', 'set', 'transfer', $transfer);
		if ($transfer == 'local')
			$config->edit('backup', 'set', 'storagepath', $storagepath);
		else
		{
			$config->edit('backup', 'set', 'ftp_server', $server);
			$config->edit('backup', 'set', 'ftp_path', $serverpath);
			$config->edit('backup', 'set', 'ftp_user', $user);
			$config->edit('backup', 'set', 'ftp_pass', $password);
			$config->edit('backup', 'set', 'createftpdatedirs', ($datedirs == "yes")? "yes" : "no");
		}

		if ($time == '')
			$time = '01:00';

		if ($frequency == 'never')
			$config->edit('backup', 'disable', '');
		else
		{
			if (($timeparts = explode(':', $time)) === false)
				throw new Exception('Invalid time provided.');

			list($hour, $minute) = $timeparts;

			if (!is_numeric($hour) || ($hour < 0) || ($hour > 23)
				|| !is_numeric($minute) || ($minute < 0) || ($minute > 59))
			{
				throw new Exception('Invalid time provided.');
			}

			switch ($frequency)
			{
				case 'yearly':
					$line = "{$minute} {$hour} 1 1 *";
					break;

				case 'monthly':
					$line = "{$minute} {$hour} 1 * *";
					break;

				case 'weekly':
					$line = "{$minute} {$hour} * * 0";
					break;

				case 'daily':
					$line = "{$minute} {$hour} * * *";
					break;

				default:
					throw new Exception('Invalid backup frequency specified.');
			}

			$line_base64 = base64_encode($line);
			$config->edit('backup', 'schedule', $line_base64);
		}
	}
}
