<?php defined('SYSPATH') OR die('No direct access allowed.');

class Network_Model
{
	private $config;

	public function __construct()
	{
		$this->config = new Settings();
	}

	public function read()
	{
		$settings = array();
		$eths = array();
		$bonds = array();

		$settings['DNS'] = array('type' => 'dns');
		$settings['DNS']['host'] = $this->config->read('network', 'hostname');
		$dns = explode("\n", $this->config->read('network', 'dns'));
		$settings['DNS']['dns1'] = isset($dns[0]) ? $dns[0] : '';
		$settings['DNS']['dns2'] = isset($dns[1]) ? $dns[1] : '';
		$settings['DNS']['dns3'] = isset($dns[2]) ? $dns[2] : '';

		$ifaces = explode("\n", $this->config->read('network', 'iface-list'));
		if (count($ifaces) && $ifaces[0] !== '')
			foreach ($ifaces as $iface)
			{
				if ($this->config->read('network', 'iface-isbond', $iface) == 'yes')
					$bonds[] = $iface;
				else
					$eths[] = $iface;

				if ($this->config->read('network', 'iface-type', $iface) == 'dhcp')
					$settings[$iface] = array('type' => 'dhcp');
				else
				{
					$settings[$iface] = array('type' => 'eth');
					
					foreach( array('ip', 'netmask', 'gateway') as $key )
					{
						$val = $this->config->read('network', 'iface-info', "$iface $key");
						$settings[$iface][$key] = isset($val)? $val : "";
					}
				}
			}
		foreach ($bonds as $master)
		{
			$slaves = explode("\n", $this->config->read('network', 'iface-list-slaves', $master));
			foreach ($slaves as $slave)
			{
				$settings[$slave]['type'] = 'bond';
				$settings[$slave]['bond'] = $master;
			}
		}

		return array($settings, $eths, $bonds);
	}

	public function edit($settings, $eths, $bonds)
	{
		$this->config->begin();
		$this->config->edit('network', 'prepare', '');

		if ($settings['DNS']['type'] == 'dns')
		{
			$this->config->edit('network', 'hostname', $settings['DNS']['host']);
			$this->config->edit('network', 'dns-all',
				base64_encode("{$settings['DNS']['dns1']} {$settings['DNS']['dns2']} {$settings['DNS']['dns3']}"));
		}
		else
		{
			; // what to do with DHCP?
		}

		foreach ($bonds as $i)
			if ($settings[$i]['type'] == 'dhcp')
				$this->config->edit('network', 'iface-dhcp', $i, $i);
			else
				$this->config->edit('network', 'iface-manual', $i,
					"{$i} {$settings[$i]["ip"]} {$settings[$i]['netmask']} {$settings[$i]['gateway']}");

		foreach ($eths as $i)
			switch ($settings[$i]['type'])
			{
				case 'dhcp':
					$this->config->edit('network', 'iface-dhcp', $i, $i);
					break;

				case 'eth':
					$this->config->edit('network', 'iface-manual', $i,
						"{$i} {$settings[$i]['ip']} {$settings[$i]['netmask']} {$settings[$i]['gateway']}");
					break;

				case 'bond':
					$this->config->edit('network', 'iface-bond', $settings[$i]['bond'], $i);
			}

		foreach ($bonds as $i)
			$this->config->edit('network', 'set_bond_opt', $i);
			
		$this->config->apply('network');
		$this->config->end();
	}
}
